#------------------------------------------------------------------------------
# Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
#
# Portions Copyright 2007-2015, Anthony Tuininga. All rights reserved.
#
# Portions Copyright 2001-2007, Computronix (Canada) Ltd., Edmonton, Alberta,
# Canada. All rights reserved.
#------------------------------------------------------------------------------

"""Runs all defined unit tests."""

import cx_Oracle
import os
import sys
import TestEnv
import unittest

# display version of cx_Oracle and Oracle client for which tests are being run
print("Running tests for cx_Oracle version", cx_Oracle.version,
        "built at", cx_Oracle.buildtime)
print("File:", cx_Oracle.__file__)
print("Client Version:", ".".join(str(i) for i in cx_Oracle.clientversion()))
sys.stdout.flush()

# verify that we can connect to the database and display database version
connection = TestEnv.GetConnection()
print("Server Version:", connection.version)
sys.stdout.flush()

# define test cases to run
moduleNames = [
    "Module",
    "Connection",
    "Cursor",
    "CursorVar",
    "DateTimeVar",
    "DbTypes",
    "DMLReturning",
    "Error",
    "IntervalVar",
    "LobVar",
    "LongVar",
    "NCharVar",
    "NumberVar",
    "ObjectVar",
    "SessionPool",
    "StringVar",
    "TimestampVar",
    "AQ",
    "BulkAQ",
    "Rowid",
    "Subscription"
]
clientVersion = cx_Oracle.clientversion()
if clientVersion[:2] >= (12, 1):
    moduleNames.append("BooleanVar")
    moduleNames.append("Features12_1")
if clientVersion[:2] >= (18, 3):
    moduleNames.append("SodaDatabase")
    moduleNames.append("SodaCollection")

# run all test cases
failures = []
loader = unittest.TestLoader()
runner = unittest.TextTestRunner(verbosity = 2)
for name in moduleNames:
    print()
    print("Running tests in", name)
    tests = loader.loadTestsFromName(name + ".TestCase")
    result = runner.run(tests)
    if not result.wasSuccessful():
        failures.append(name)
if failures:
    print("***** Some tests in the following modules failed. *****")
    for name in failures:
        print("      %s" % name)
    sys.exit(1)



#------------------------------------------------------------------------------
# Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

"""Module for testing AQ Bulk enqueue/dequeue."""

import TestEnv

import cx_Oracle
import decimal
import threading

RAW_QUEUE_NAME = "TEST_RAW_QUEUE"
RAW_PAYLOAD_DATA = [
    "The first message",
    "The second message",
    "The third message",
    "The fourth message",
    "The fifth message",
    "The sixth message",
    "The seventh message",
    "The eighth message",
    "The ninth message",
    "The tenth message",
    "The eleventh message",
    "The twelfth and final message"
]

class TestCase(TestEnv.BaseTestCase):

    def __deqInThread(self, results):
        connection = TestEnv.GetConnection(threaded=True)
        queue = connection.queue(RAW_QUEUE_NAME)
        queue.deqOptions.wait = 10
        queue.deqOptions.navigation = cx_Oracle.DEQ_FIRST_MSG
        while len(results) < len(RAW_PAYLOAD_DATA):
            messages = queue.deqMany(5)
            if not messages:
                break
            for m in messages:
                results.append(m.payload.decode(connection.encoding))
        connection.commit()

    def __getAndClearRawQueue(self):
        queue = self.connection.queue(RAW_QUEUE_NAME)
        queue.deqOptions.wait = cx_Oracle.DEQ_NO_WAIT
        queue.deqOptions.navigation = cx_Oracle.DEQ_FIRST_MSG
        while queue.deqOne():
            pass
        self.connection.commit()
        return queue

    def testEnqAndDeq(self):
        "test bulk enqueue and dequeue"
        queue = self.__getAndClearRawQueue()
        messages = [self.connection.msgproperties(payload=d) \
                for d in RAW_PAYLOAD_DATA]
        queue.enqMany(messages)
        messages = queue.deqMany(len(RAW_PAYLOAD_DATA))
        data = [m.payload.decode(self.connection.encoding) for m in messages]
        self.connection.commit()
        self.assertEqual(data, RAW_PAYLOAD_DATA)

    def testDequeueEmpty(self):
        "test empty bulk dequeue"
        queue = self.__getAndClearRawQueue()
        messages = queue.deqMany(5)
        self.connection.commit()
        self.assertEqual(messages, [])

    def testDeqWithWait(self):
        "test bulk dequeue with wait"
        queue = self.__getAndClearRawQueue()
        results = []
        thread = threading.Thread(target=self.__deqInThread, args=(results,))
        thread.start()
        messages = [self.connection.msgproperties(payload=d) \
                for d in RAW_PAYLOAD_DATA]
        queue.enqOptions.visibility = cx_Oracle.ENQ_IMMEDIATE
        queue.enqMany(messages)
        thread.join()
        self.assertEqual(results, RAW_PAYLOAD_DATA)

    def testEnqAndDeqMultipleTimes(self):
        "test enqueue and dequeue multiple times"
        queue = self.__getAndClearRawQueue()
        dataToEnqueue = RAW_PAYLOAD_DATA
        for num in (2, 6, 4):
            messages = [self.connection.msgproperties(payload=d) \
                    for d in dataToEnqueue[:num]]
            dataToEnqueue = dataToEnqueue[num:]
            queue.enqMany(messages)
        self.connection.commit()
        allData = []
        for num in (3, 5, 10):
            messages = queue.deqMany(num)
            allData.extend(m.payload.decode(self.connection.encoding) \
                    for m in messages)
        self.connection.commit()
        self.assertEqual(allData, RAW_PAYLOAD_DATA)

    def testEnqAndDeqVisibility(self):
        "test visibility option for enqueue and dequeue"
        queue = self.__getAndClearRawQueue()

        # first test with ENQ_ON_COMMIT (commit required)
        queue.enqOptions.visibility = cx_Oracle.ENQ_ON_COMMIT
        props1 = self.connection.msgproperties(payload="A first message")
        props2 = self.connection.msgproperties(payload="A second message")
        queue.enqMany([props1, props2])
        otherConnection = TestEnv.GetConnection()
        otherQueue = otherConnection.queue(RAW_QUEUE_NAME)
        otherQueue.deqOptions.wait = cx_Oracle.DEQ_NO_WAIT
        otherQueue.deqOptions.visibility = cx_Oracle.DEQ_ON_COMMIT
        messages = otherQueue.deqMany(5)
        self.assertEqual(len(messages), 0)
        self.connection.commit()
        messages = otherQueue.deqMany(5)
        self.assertEqual(len(messages), 2)
        otherConnection.rollback()

        # second test with ENQ_IMMEDIATE (no commit required)
        queue.enqOptions.visibility = cx_Oracle.ENQ_IMMEDIATE
        otherQueue.deqOptions.visibility = cx_Oracle.DEQ_IMMEDIATE
        queue.enqMany([props1, props2])
        messages = otherQueue.deqMany(5)
        self.assertEqual(len(messages), 4)
        otherConnection.rollback()
        messages = otherQueue.deqMany(5)
        self.assertEqual(len(messages), 0)

if __name__ == "__main__":
    TestEnv.RunTestCases()



#------------------------------------------------------------------------------
# Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# ShardingNumberKey.py
#   This script demonstrates how to use sharding keys with a sharded database.
# The sample schema provided does not include support for running this demo. A
# sharded database must first be created. Information on how to create a
# sharded database can be found in the documentation:
# https://www.oracle.com/pls/topic/lookup?ctx=dblatest&id=SHARD
#
# This script requires cx_Oracle 6.1 and higher but it is recommended to use
# cx_Oracle 7.3 and higher in order to avoid a set of known issues when using
# sharding capabilities.
#------------------------------------------------------------------------------

import cx_Oracle
import SampleEnv

pool = cx_Oracle.SessionPool(SampleEnv.GetMainUser(),
        SampleEnv.GetMainPassword(), SampleEnv.GetConnectString(), min=1,
        max=5, increment=1)

def ConnectAndDisplay(shardingKey):
    print("Connecting with sharding key:", shardingKey)
    with pool.acquire(shardingkey=[shardingKey]) as conn:
        cursor = conn.cursor()
        cursor.execute("select sys_context('userenv', 'db_name') from dual")
        name, = cursor.fetchone()
        print("--> connected to database", name)

ConnectAndDisplay(100)
ConnectAndDisplay(167)



#------------------------------------------------------------------------------
# Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

"""Module for testing Simple Oracle Document Access (SODA) Collections"""

import TestEnv

import cx_Oracle

class TestCase(TestEnv.BaseTestCase):

    def __testSkip(self, coll, numToSkip, expectedContent):
        filterSpec = {'$orderby': [{'path': 'name', 'order': 'desc'}]}
        doc = coll.find().filter(filterSpec).skip(numToSkip).getOne()
        content = doc.getContent() if doc is not None else None
        self.assertEqual(content, expectedContent)

    def testInvalidJson(self):
        "test inserting invalid JSON value into SODA collection"
        invalidJson = "{testKey:testValue}"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoInvalidJSON")
        doc = sodaDatabase.createDocument(invalidJson)
        self.assertRaises(cx_Oracle.IntegrityError, coll.insertOne, doc)
        coll.drop()

    def testInsertDocuments(self):
        "test inserting documents into a SODA collection"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoInsertDocs")
        coll.find().remove()
        valuesToInsert = [
            { "name" : "George", "age" : 47 },
            { "name" : "Susan", "age" : 39 },
            { "name" : "John", "age" : 50 },
            { "name" : "Jill", "age" : 54 }
        ]
        insertedKeys = []
        for value in valuesToInsert:
            doc = coll.insertOneAndGet(value)
            insertedKeys.append(doc.key)
        self.connection.commit()
        self.assertEqual(coll.find().count(), len(valuesToInsert))
        for key, value in zip(insertedKeys, valuesToInsert):
            doc = coll.find().key(key).getOne()
            self.assertEqual(doc.getContent(), value)
        coll.drop()

    def testSkipDocuments(self):
        "test skipping documents in a SODA collection"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoSkipDocs")
        coll.find().remove()
        valuesToInsert = [
            { "name" : "Anna", "age" : 62 },
            { "name" : "Mark", "age" : 37 },
            { "name" : "Martha", "age" : 43 },
            { "name" : "Matthew", "age" : 28 }
        ]
        for value in valuesToInsert:
            coll.insertOne(value)
        self.connection.commit()
        self.__testSkip(coll, 0, valuesToInsert[3])
        self.__testSkip(coll, 1, valuesToInsert[2])
        self.__testSkip(coll, 3, valuesToInsert[0])
        self.__testSkip(coll, 4, None)
        self.__testSkip(coll, 125, None)

    def testReplaceDocument(self):
        "test replace documents in SODA collection"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoReplaceDoc")
        coll.find().remove()
        content = {'name': 'John', 'address': {'city': 'Sydney'}}
        doc = coll.insertOneAndGet(content)
        newContent = {'name': 'John', 'address': {'city':'Melbourne'}}
        coll.find().key(doc.key).replaceOne(newContent)
        self.connection.commit()
        self.assertEqual(coll.find().key(doc.key).getOne().getContent(),
                newContent)
        coll.drop()

    def testSearchDocumentsWithContent(self):
        "test search documents with content using $like and $regex"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoSearchDocContent")
        coll.find().remove()
        data = [
            {'name': 'John', 'address': {'city': 'Bangalore'}},
            {'name': 'Johnson', 'address': {'city': 'Banaras'}},
            {'name': 'Joseph', 'address': {'city': 'Bangalore'}},
            {'name': 'Jibin', 'address': {'city': 'Secunderabad'}},
            {'name': 'Andrew', 'address': {'city': 'Hyderabad'}},
            {'name': 'Matthew', 'address': {'city': 'Mumbai'}}
        ]
        for value in data:
            coll.insertOne(value)
        self.connection.commit()
        filterSpecs = [
            ({'name': {'$like': 'And%'}}, 1),
            ({'name': {'$like': 'J%n'}}, 3),
            ({'name': {'$like': '%hn%'}}, 2),
            ({'address.city': {'$like': 'Ban%'}}, 3),
            ({'address.city': {'$like': '%bad'}}, 2),
            ({'address.city': {'$like': 'Hyderabad'}}, 1),
            ({'address.city': {'$like': 'China%'}}, 0),
            ({'name': {'$regex': 'Jo.*'}}, 3),
            ({'name': {'$regex': '.*[ho]n'}}, 2),
            ({'name': {'$regex': 'J.*h'}}, 1),
            ({'address.city': {'$regex': 'Ba.*'}}, 3),
            ({'address.city': {'$regex': '.*bad'}}, 2),
            ({'address.city': {'$regex': 'Hyderabad'}}, 1),
            ({'name': {'$regex': 'Js.*n'}}, 0)
        ]
        for filterSpec, expectedCount in filterSpecs:
            self.assertEqual(coll.find().filter(filterSpec).count(),
                    expectedCount, filterSpec)
        coll.drop()

    def testDocumentRemove(self):
        "test removing documents"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoRemoveDocs")
        coll.find().remove()
        data = [
            {'name': 'John', 'address': {'city': 'Bangalore'}},
            {'name': 'Johnson', 'address': {'city': 'Banaras'}},
            {'name': 'Joseph', 'address': {'city': 'Mangalore'}},
            {'name': 'Jibin', 'address': {'city': 'Secunderabad'}},
            {'name': 'Andrew', 'address': {'city': 'Hyderabad'}},
            {'name': 'Matthew', 'address': {'city': 'Mumbai'}}
        ]
        docs = [coll.insertOneAndGet(v) for v in data]
        coll.find().key(docs[3].key).remove()
        self.assertEqual(coll.find().count(), len(data) - 1)
        searchResults = coll.find().filter({'name': {'$like': 'Jibin'}})
        self.assertEqual(searchResults.count(), 0)
        coll.find().filter({'name': {'$like': 'John%'}}).remove()
        self.assertEqual(coll.find().count(), len(data) - 3)
        coll.find().filter({'name': {'$regex': 'J.*'}}).remove()
        self.assertEqual(coll.find().count(), len(data) - 4)
        self.connection.commit()
        coll.drop()

    def testCreateAndDropIndex(self):
        "test create and drop Index"
        indexName = "cxoTestIndexes_ix_1"
        indexSpec =  {
            'name': indexName,
            'fields': [
                {
                    'path': 'address.city',
                    'datatype': 'string',
                    'order': 'asc'
                }
            ]
        }
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoTestIndexes")
        coll.find().remove()
        self.connection.commit()
        coll.dropIndex(indexName)
        coll.createIndex(indexSpec)
        self.assertRaises(cx_Oracle.DatabaseError, coll.createIndex, indexSpec)
        self.assertEqual(coll.dropIndex(indexName), True)
        self.assertEqual(coll.dropIndex(indexName), False)
        coll.drop()

    def testGetDocuments(self):
        "test getting documents from Collection"
        self.connection.autocommit = True
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoTestGetDocs")
        coll.find().remove()
        data = [
            {'name': 'John', 'address': {'city': 'Bangalore'}},
            {'name': 'Johnson', 'address': {'city': 'Banaras'}},
            {'name': 'Joseph', 'address': {'city': 'Mangalore'}},
            {'name': 'Jibin', 'address': {'city': 'Secunderabad'}},
            {'name': 'Andrew', 'address': {'city': 'Hyderabad'}}
        ]
        insertedKeys = list(sorted(coll.insertOneAndGet(v).key for v in data))
        fetchedKeys = list(sorted(d.key for d in coll.find().getDocuments()))
        self.assertEqual(fetchedKeys, insertedKeys)
        coll.drop()

    def testCursor(self):
        "test fetching documents from a cursor"
        self.connection.autocommit = True
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoFindViaCursor")
        coll.find().remove()
        data = [
            {'name': 'John', 'address': {'city': 'Bangalore'}},
            {'name': 'Johnson', 'address': {'city': 'Banaras'}},
            {'name': 'Joseph', 'address': {'city': 'Mangalore'}},
        ]
        insertedKeys = list(sorted(coll.insertOneAndGet(v).key for v in data))
        fetchedKeys = list(sorted(d.key for d in coll.find().getCursor()))
        self.assertEqual(fetchedKeys, insertedKeys)
        coll.drop()

    def testMultipleDocumentRemove(self):
        "test removing multiple documents using multiple keys"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoRemoveMultipleDocs")
        coll.find().remove()
        data = [
            {'name': 'John', 'address': {'city': 'Bangalore'}},
            {'name': 'Johnson', 'address': {'city': 'Banaras'}},
            {'name': 'Joseph', 'address': {'city': 'Mangalore'}},
            {'name': 'Jibin', 'address': {'city': 'Secunderabad'}},
            {'name': 'Andrew', 'address': {'city': 'Hyderabad'}},
            {'name': 'Matthew', 'address': {'city': 'Mumbai'}}
        ]
        docs = [coll.insertOneAndGet(v) for v in data]
        keys = [docs[i].key for i in (1, 3, 5)]
        numRemoved = coll.find().keys(keys).remove()
        self.assertEqual(numRemoved, len(keys))
        self.assertEqual(coll.find().count(), len(data) - len(keys))
        self.connection.commit()
        coll.drop()

    def testDocumentVersion(self):
        "test using version to get documents and remove them"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoDocumentVersion")
        coll.find().remove()
        content = {'name': 'John', 'address': {'city': 'Bangalore'}}
        insertedDoc = coll.insertOneAndGet(content)
        key = insertedDoc.key
        version = insertedDoc.version
        doc = coll.find().key(key).version(version).getOne()
        self.assertEqual(doc.getContent(), content)
        newContent = {'name': 'James', 'address': {'city': 'Delhi'}}
        replacedDoc = coll.find().key(key).replaceOneAndGet(newContent)
        newVersion = replacedDoc.version
        doc = coll.find().key(key).version(version).getOne()
        self.assertEqual(doc, None)
        doc = coll.find().key(key).version(newVersion).getOne()
        self.assertEqual(doc.getContent(), newContent)
        self.assertEqual(coll.find().key(key).version(version).remove(), 0)
        self.assertEqual(coll.find().key(key).version(newVersion).remove(), 1)
        self.assertEqual(coll.find().count(), 0)
        self.connection.commit()
        coll.drop()

    def testGetCursor(self):
        "test keys with GetCursor"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoKeysWithGetCursor")
        coll.find().remove()
        data = [
            {'name': 'John', 'address': {'city': 'Bangalore'}},
            {'name': 'Johnson', 'address': {'city': 'Banaras'}},
            {'name': 'Joseph', 'address': {'city': 'Mangalore'}},
            {'name': 'Jibin', 'address': {'city': 'Secunderabad'}},
            {'name': 'Andrew', 'address': {'city': 'Hyderabad'}},
            {'name': 'Matthew', 'address': {'city': 'Mumbai'}}
        ]
        docs = [coll.insertOneAndGet(v) for v in data]
        keys = [docs[i].key for i in (2, 4, 5)]
        fetchedKeys = [d.key for d in coll.find().keys(keys).getCursor()]
        self.assertEqual(list(sorted(fetchedKeys)), list(sorted(keys)))
        self.connection.commit()
        coll.drop()

    def testCreatedOn(self):
        "test createdOn attribute of Document"
        sodaDatabase = self.getSodaDatabase()
        coll = sodaDatabase.createCollection("cxoCreatedOn")
        coll.find().remove()
        data = {'name': 'John', 'address': {'city': 'Bangalore'}}
        doc = coll.insertOneAndGet(data)
        self.assertEqual(doc.createdOn, doc.lastModified)

    def testSodaTruncate(self):
        "test Soda truncate"
        sodaDatabase = self.getSodaDatabase(minclient=(20,1))
        coll = sodaDatabase.createCollection("cxoTruncateDocs")
        coll.find().remove()
        valuesToInsert = [
            { "name" : "George", "age" : 47 },
            { "name" : "Susan", "age" : 39 },
            { "name" : "John", "age" : 50 },
            { "name" : "Jill", "age" : 54 }
        ]
        for value in valuesToInsert:
            coll.insertOne(value)
        self.connection.commit()
        self.assertEqual(coll.find().count(), len(valuesToInsert))
        coll.truncate()
        self.assertEqual(coll.find().count(), 0)
        coll.drop()

if __name__ == "__main__":
    TestEnv.RunTestCases()




#------------------------------------------------------------------------------
# Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# SetupTest.py
#
# Creates users and populates their schemas with the tables and packages
# necessary for the cx_Oracle test suite.
#------------------------------------------------------------------------------

import cx_Oracle

import TestEnv
import DropTest

# connect as administrative user (usually SYSTEM or ADMIN)
conn = cx_Oracle.connect(TestEnv.GetAdminConnectString())

# drop existing users and editions, if applicable
DropTest.DropTests(conn)

# create test schemas
print("Creating test schemas...")
TestEnv.RunSqlScript(conn, "SetupTest",
        main_user = TestEnv.GetMainUser(),
        main_password = TestEnv.GetMainPassword(),
        proxy_user = TestEnv.GetProxyUser(),
        proxy_password = TestEnv.GetProxyPassword())
print("Done.")




#------------------------------------------------------------------------------
# Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# RefCursor.py
#   Demonstrates the use of REF cursors with cx_Oracle.
#------------------------------------------------------------------------------

import cx_Oracle
import SampleEnv

connection = cx_Oracle.connect(SampleEnv.GetMainConnectString())
cursor = connection.cursor()

refCursor = connection.cursor()
cursor.callproc("myrefcursorproc", (2, 6, refCursor))
print("Rows between 2 and 6:")
for row in refCursor:
    print(row)
print()

refCursor = connection.cursor()
cursor.callproc("myrefcursorproc", (8, 9, refCursor))
print("Rows between 8 and 9:")
for row in refCursor:
    print(row)
print()




#------------------------------------------------------------------------------
# Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# SetupSamples.py
#
# Creates users and populates their schemas with the tables and packages
# necessary for the cx_Oracle samples. An edition is also created for the
# demonstration of PL/SQL editioning.
#------------------------------------------------------------------------------

import cx_Oracle

import SampleEnv
import DropSamples

# connect as administrative user (usually SYSTEM or ADMIN)
conn = cx_Oracle.connect(SampleEnv.GetAdminConnectString())

# drop existing users and editions, if applicable
DropSamples.DropSamples(conn)

# create sample schema and edition
print("Creating sample schemas and edition...")
SampleEnv.RunSqlScript(conn, "SetupSamples",
        main_user = SampleEnv.GetMainUser(),
        main_password = SampleEnv.GetMainPassword(),
        edition_user = SampleEnv.GetEditionUser(),
        edition_password = SampleEnv.GetEditionPassword(),
        edition_name = SampleEnv.GetEditionName())
print("Done.")




#------------------------------------------------------------------------------
# Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

"""
Module for testing comparisons with database types and API types. This also
contains tests for comparisons with database types and variable types, for
backwards compatibility.
"""

import TestEnv

import cx_Oracle

class TestCase(TestEnv.BaseTestCase):

    def __testCompare(self, dbType, apiType):
        self.assertEqual(dbType, dbType)
        self.assertEqual(dbType, apiType)
        self.assertEqual(apiType, dbType)
        self.assertNotEqual(dbType, 5)
        self.assertNotEqual(dbType, cx_Oracle.DB_TYPE_OBJECT)

    def testBfile(self):
        "test cx_Oracle.DB_TYPE_BFILE comparisons"
        self.assertEqual(cx_Oracle.DB_TYPE_BFILE, cx_Oracle.BFILE)

    def testBinaryDouble(self):
        "test cx_Oracle.DB_TYPE_BINARY_DOUBLE comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_BINARY_DOUBLE, cx_Oracle.NUMBER)
        self.assertEqual(cx_Oracle.DB_TYPE_BINARY_DOUBLE,
                cx_Oracle.NATIVE_FLOAT)

    def testBinaryFloat(self):
        "test cx_Oracle.DB_TYPE_BINARY_FLOAT comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_BINARY_FLOAT, cx_Oracle.NUMBER)

    def testBinaryInteger(self):
        "test cx_Oracle.DB_TYPE_BINARY_INTEGER comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_BINARY_INTEGER, cx_Oracle.NUMBER)
        self.assertEqual(cx_Oracle.DB_TYPE_BINARY_INTEGER,
                cx_Oracle.NATIVE_INT)

    def testBlob(self):
        "test cx_Oracle.DB_TYPE_BLOB comparisons"
        self.assertEqual(cx_Oracle.DB_TYPE_BLOB, cx_Oracle.BLOB)

    def testBoolean(self):
        "test cx_Oracle.DB_TYPE_BOOLEAN comparisons"
        self.assertEqual(cx_Oracle.DB_TYPE_BOOLEAN, cx_Oracle.BOOLEAN)

    def testChar(self):
        "test cx_Oracle.DB_TYPE_CHAR comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_CHAR, cx_Oracle.STRING)
        self.assertEqual(cx_Oracle.DB_TYPE_CHAR, cx_Oracle.FIXED_CHAR)

    def testClob(self):
        "test cx_Oracle.DB_TYPE_CLOB comparisons"
        self.assertEqual(cx_Oracle.DB_TYPE_CLOB, cx_Oracle.CLOB)

    def testCursor(self):
        "test cx_Oracle.DB_TYPE_CURSOR comparisons"
        self.assertEqual(cx_Oracle.DB_TYPE_CURSOR, cx_Oracle.CURSOR)

    def testDate(self):
        "test cx_Oracle.DB_TYPE_DATE comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_DATE, cx_Oracle.DATETIME)

    def testIntervalDS(self):
        "test cx_Oracle.DB_TYPE_INTERVAL_DS comparisons"
        self.assertEqual(cx_Oracle.DB_TYPE_INTERVAL_DS, cx_Oracle.INTERVAL)

    def testLong(self):
        "test cx_Oracle.DB_TYPE_LONG comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_LONG, cx_Oracle.STRING)
        self.assertEqual(cx_Oracle.DB_TYPE_LONG, cx_Oracle.LONG_STRING)

    def testLongRaw(self):
        "test cx_Oracle.DB_TYPE_LONG_RAW comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_LONG_RAW, cx_Oracle.BINARY)
        self.assertEqual(cx_Oracle.DB_TYPE_LONG_RAW, cx_Oracle.LONG_BINARY)

    def testNchar(self):
        "test cx_Oracle.DB_TYPE_NCHAR comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_NCHAR, cx_Oracle.STRING)
        self.assertEqual(cx_Oracle.DB_TYPE_NCHAR, cx_Oracle.FIXED_NCHAR)

    def testNclob(self):
        "test cx_Oracle.DB_TYPE_NCLOB comparisons"
        self.assertEqual(cx_Oracle.DB_TYPE_NCLOB, cx_Oracle.NCLOB)

    def testNumber(self):
        "test cx_Oracle.DB_TYPE_NUMBER comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_NUMBER, cx_Oracle.NUMBER)

    def testNvarchar(self):
        "test cx_Oracle.DB_TYPE_NVARCHAR comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_NVARCHAR, cx_Oracle.STRING)
        self.assertEqual(cx_Oracle.DB_TYPE_NVARCHAR, cx_Oracle.NCHAR)

    def testObject(self):
        "test cx_Oracle.DB_TYPE_OBJECT comparisons"
        self.assertEqual(cx_Oracle.DB_TYPE_OBJECT, cx_Oracle.OBJECT)

    def testRaw(self):
        "test cx_Oracle.DB_TYPE_RAW comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_RAW, cx_Oracle.BINARY)

    def testRowid(self):
        "test cx_Oracle.DB_TYPE_ROWID comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_ROWID, cx_Oracle.ROWID)

    def testTimestamp(self):
        "test cx_Oracle.DB_TYPE_TIMESTAMP comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_TIMESTAMP, cx_Oracle.DATETIME)
        self.assertEqual(cx_Oracle.DB_TYPE_TIMESTAMP, cx_Oracle.TIMESTAMP)

    def testTimestampLTZ(self):
        "test cx_Oracle.DB_TYPE_TIMESTAMP_LTZ comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_TIMESTAMP_LTZ, cx_Oracle.DATETIME)

    def testTimestampTZ(self):
        "test cx_Oracle.DB_TYPE_TIMESTAMP_TZ comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_TIMESTAMP_TZ, cx_Oracle.DATETIME)

    def testVarchar(self):
        "test cx_Oracle.DB_TYPE_VARCHAR comparisons"
        self.__testCompare(cx_Oracle.DB_TYPE_VARCHAR, cx_Oracle.STRING)


if __name__ == "__main__":
    TestEnv.RunTestCases()



#------------------------------------------------------------------------------
# Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

"""Module for testing Simple Oracle Document Access (SODA) Database"""

import TestEnv

import cx_Oracle
import json

class TestCase(TestEnv.BaseTestCase):

    def __dropExistingCollections(self, sodaDatabase):
        for name in sodaDatabase.getCollectionNames():
            sodaDatabase.openCollection(name).drop()

    def __verifyDocument(self, doc, rawContent, strContent=None, content=None,
            key=None, mediaType='application/json'):
        self.assertEqual(doc.getContentAsBytes(), rawContent)
        if strContent is not None:
            self.assertEqual(doc.getContentAsString(), strContent)
        if content is not None:
            self.assertEqual(doc.getContent(), content)
        self.assertEqual(doc.key, key)
        self.assertEqual(doc.mediaType, mediaType)

    def testCreateDocumentWithJson(self):
        "test creating documents with JSON data"
        sodaDatabase = self.getSodaDatabase()
        val = {"testKey1" : "testValue1", "testKey2" : "testValue2" }
        strVal = json.dumps(val)
        bytesVal = strVal.encode("UTF-8")
        key = "MyKey"
        mediaType = "text/plain"
        doc = sodaDatabase.createDocument(val)
        self.__verifyDocument(doc, bytesVal, strVal, val)
        doc = sodaDatabase.createDocument(strVal, key)
        self.__verifyDocument(doc, bytesVal, strVal, val, key)
        doc = sodaDatabase.createDocument(bytesVal, key, mediaType)
        self.__verifyDocument(doc, bytesVal, strVal, val, key, mediaType)

    def testCreateDocumentWithRaw(self):
        "test creating documents with raw data"
        sodaDatabase = self.getSodaDatabase()
        val = b"<html/>"
        key = "MyRawKey"
        mediaType = "text/html"
        doc = sodaDatabase.createDocument(val)
        self.__verifyDocument(doc, val)
        doc = sodaDatabase.createDocument(val, key)
        self.__verifyDocument(doc, val, key=key)
        doc = sodaDatabase.createDocument(val, key, mediaType)
        self.__verifyDocument(doc, val, key=key, mediaType=mediaType)

    def testGetCollectionNames(self):
        "test getting collection names from the database"
        sodaDatabase = self.getSodaDatabase()
        self.__dropExistingCollections(sodaDatabase)
        self.assertEqual(sodaDatabase.getCollectionNames(), [])
        names = ["zCol", "dCol", "sCol", "aCol", "gCol"]
        sortedNames = list(sorted(names))
        for name in names:
            sodaDatabase.createCollection(name)
        self.assertEqual(sodaDatabase.getCollectionNames(), sortedNames)
        self.assertEqual(sodaDatabase.getCollectionNames(limit=2),
                sortedNames[:2])
        self.assertEqual(sodaDatabase.getCollectionNames("a"), sortedNames)
        self.assertEqual(sodaDatabase.getCollectionNames("C"), sortedNames)
        self.assertEqual(sodaDatabase.getCollectionNames("b", limit=3),
                sortedNames[1:4])
        self.assertEqual(sodaDatabase.getCollectionNames("z"),
                sortedNames[-1:])

    def testOpenCollection(self):
        "test opening a collection"
        sodaDatabase = self.getSodaDatabase()
        self.__dropExistingCollections(sodaDatabase)
        coll = sodaDatabase.openCollection("CollectionThatDoesNotExist")
        self.assertEqual(coll, None)
        createdColl = sodaDatabase.createCollection("cxoTestOpenCollection")
        coll = sodaDatabase.openCollection(createdColl.name)
        self.assertEqual(coll.name, createdColl.name)
        coll.drop()

    def testRepr(self):
        "test SodaDatabase representation"
        con1 = self.connection
        con2 = TestEnv.GetConnection()
        sodaDatabase1 = self.getSodaDatabase()
        sodaDatabase2 = con1.getSodaDatabase()
        sodaDatabase3 = con2.getSodaDatabase()
        self.assertEqual(str(sodaDatabase1), str(sodaDatabase2))
        self.assertEqual(str(sodaDatabase2), str(sodaDatabase3))

    def testNegative(self):
        "test negative cases for SODA database methods"
        sodaDatabase = self.getSodaDatabase()
        self.assertRaises(TypeError, sodaDatabase.createCollection)
        self.assertRaises(TypeError, sodaDatabase.createCollection, 1)
        self.assertRaises(cx_Oracle.DatabaseError,
                sodaDatabase.createCollection, None)
        self.assertRaises(TypeError, sodaDatabase.getCollectionNames, 1)

if __name__ == "__main__":
    TestEnv.RunTestCases()



#------------------------------------------------------------------------------
# Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
#
# Portions Copyright 2007-2015, Anthony Tuininga. All rights reserved.
#
# Portions Copyright 2001-2007, Computronix (Canada) Ltd., Edmonton, Alberta,
# Canada. All rights reserved.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# MultiConsumerAQ.py
#   This script demonstrates how to use multi-consumer
# advanced queuing using cx_Oracle. It makes use of a RAW queue
# created in the sample setup.
#
# This script requires cx_Oracle 7.2 and higher.
#------------------------------------------------------------------------------

import cx_Oracle
import SampleEnv

QUEUE_NAME = "DEMO_RAW_QUEUE_MULTI"
PAYLOAD_DATA = [
    "The first message",
    "The second message",
    "The third message",
    "The fourth and final message"
]

# connect to database
connection = cx_Oracle.connect(SampleEnv.GetMainConnectString())
cursor = connection.cursor()

# create queue
queue = connection.queue(QUEUE_NAME)
queue.deqOptions.wait = cx_Oracle.DEQ_NO_WAIT
queue.deqOptions.navigation = cx_Oracle.DEQ_FIRST_MSG

# enqueue a few messages
print("Enqueuing messages...")
for data in PAYLOAD_DATA:
    print(data)
    queue.enqOne(connection.msgproperties(payload=data))
connection.commit()
print()

# dequeue the messages for consumer A
print("Dequeuing the messages for consumer A...")
queue.deqOptions.consumername = "SUBSCRIBER_A"
while True:
    props = queue.deqOne()
    if not props:
        break
    print(props.payload.decode())
connection.commit()
print()

# dequeue the message for consumer B
print("Dequeuing the messages for consumer B...")
queue.deqOptions.consumername = "SUBSCRIBER_B"
while True:
    props = queue.deqOne()
    if not props:
        break
    print(props.payload.decode())
connection.commit()

print("\nDone.")



#------------------------------------------------------------------------------
# Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
#
# Portions Copyright 2007-2015, Anthony Tuininga. All rights reserved.
#
# Portions Copyright 2001-2007, Computronix (Canada) Ltd., Edmonton, Alberta,
# Canada. All rights reserved.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Sets the environment used by the cx_Oracle test suite. Production
# applications should consider using External Authentication to
# avoid hard coded credentials.
#
# You can set values in environment variables to bypass having the test suite
# request the information it requires.
#
#     CX_ORACLE_TEST_MAIN_USER: user used for most test cases
#     CX_ORACLE_TEST_MAIN_PASSWORD: password of user used for most test cases
#     CX_ORACLE_TEST_PROXY_USER: user for testing proxying
#     CX_ORACLE_TEST_PROXY_PASSWORD: password of user for testing proxying
#     CX_ORACLE_TEST_CONNECT_STRING: connect string for test suite
#     CX_ORACLE_TEST_ADMIN_USER: administrative user for test suite
#     CX_ORACLE_TEST_ADMIN_PASSWORD: administrative password for test suite
#
# CX_ORACLE_TEST_CONNECT_STRING can be set to an Easy Connect string, or a
# Net Service Name from a tnsnames.ora file or external naming service,
# or it can be the name of a local Oracle database instance.
#
# If cx_Oracle is using Instant Client, then an Easy Connect string is
# generally appropriate. The syntax is:
#
#   [//]host_name[:port][/service_name][:server_type][/instance_name]
#
# Commonly just the host_name and service_name are needed
# e.g. "localhost/orclpdb1" or "localhost/XEPDB1"
#
# If using a tnsnames.ora file, the file can be in a default
# location such as $ORACLE_HOME/network/admin/tnsnames.ora or
# /etc/tnsnames.ora.  Alternatively set the TNS_ADMIN environment
# variable and put the file in $TNS_ADMIN/tnsnames.ora.
#
# The administrative user for cloud databases is ADMIN and the administrative
# user for on premises databases is SYSTEM.
#------------------------------------------------------------------------------

import cx_Oracle
import getpass
import os
import sys
import unittest

# default values
DEFAULT_MAIN_USER = "pythontest"
DEFAULT_PROXY_USER = "pythontestproxy"
DEFAULT_CONNECT_STRING = "localhost/orclpdb1"

# dictionary containing all parameters; these are acquired as needed by the
# methods below (which should be used instead of consulting this dictionary
# directly) and then stored so that a value is not requested more than once
PARAMETERS = {}

def GetValue(name, label, defaultValue=""):
    value = PARAMETERS.get(name)
    if value is not None:
        return value
    envName = "CX_ORACLE_TEST_" + name
    value = os.environ.get(envName)
    if value is None:
        if defaultValue:
            label += " [%s]" % defaultValue
        label += ": "
        if defaultValue:
            value = input(label).strip()
        else:
            value = getpass.getpass(label)
        if not value:
            value = defaultValue
    PARAMETERS[name] = value
    return value

def GetMainUser():
    return GetValue("MAIN_USER", "Main User Name", DEFAULT_MAIN_USER)

def GetMainPassword():
    return GetValue("MAIN_PASSWORD", "Password for %s" % GetMainUser())

def GetProxyUser():
    return GetValue("PROXY_USER", "Proxy User Name", DEFAULT_PROXY_USER)

def GetProxyPassword():
    return GetValue("PROXY_PASSWORD", "Password for %s" % GetProxyUser())

def GetConnectString():
    return GetValue("CONNECT_STRING", "Connect String", DEFAULT_CONNECT_STRING)

def GetCharSetRatio():
    value = PARAMETERS.get("CS_RATIO")
    if value is None:
        connection = GetConnection()
        cursor = connection.cursor()
        cursor.execute("select 'X' from dual")
        col, = cursor.description
        value = col[3]
        PARAMETERS["CS_RATIO"] = value
    return value

def GetAdminConnectString():
    adminUser = GetValue("ADMIN_USER", "Administrative user", "admin")
    adminPassword = GetValue("ADMIN_PASSWORD", "Password for %s" % adminUser)
    return "%s/%s@%s" % (adminUser, adminPassword, GetConnectString())

def RunSqlScript(conn, scriptName, **kwargs):
    statementParts = []
    cursor = conn.cursor()
    replaceValues = [("&" + k + ".", v) for k, v in kwargs.items()] + \
            [("&" + k, v) for k, v in kwargs.items()]
    scriptDir = os.path.dirname(os.path.abspath(sys.argv[0]))
    fileName = os.path.join(scriptDir, "sql", scriptName + "Exec.sql")
    for line in open(fileName):
        if line.strip() == "/":
            statement = "".join(statementParts).strip()
            if statement:
                for searchValue, replaceValue in replaceValues:
                    statement = statement.replace(searchValue, replaceValue)
                try:
                    cursor.execute(statement)
                except:
                    print("Failed to execute SQL:", statement)
                    raise
            statementParts = []
        else:
            statementParts.append(line)
    cursor.execute("""
            select name, type, line, position, text
            from dba_errors
            where owner = upper(:owner)
            order by name, type, line, position""",
            owner = GetMainUser())
    prevName = prevObjType = None
    for name, objType, lineNum, position, text in cursor:
        if name != prevName or objType != prevObjType:
            print("%s (%s)" % (name, objType))
            prevName = name
            prevObjType = objType
        print("    %s/%s %s" % (lineNum, position, text))

def RunTestCases():
    unittest.main(testRunner=unittest.TextTestRunner(verbosity=2))

def GetConnection(**kwargs):
    return cx_Oracle.connect(GetMainUser(), GetMainPassword(),
            GetConnectString(), encoding="UTF-8", nencoding="UTF-8", **kwargs)

def GetPool(user=None, password=None, **kwargs):
    if user is None:
        user = GetMainUser()
    if password is None:
        password = GetMainPassword()
    return cx_Oracle.SessionPool(user, password, GetConnectString(),
            encoding="UTF-8", nencoding="UTF-8", **kwargs)

def GetClientVersion():
    return cx_Oracle.clientversion()

class BaseTestCase(unittest.TestCase):

    def getSodaDatabase(self, minclient=(18, 3), minserver=(18, 0),
            message="not supported with this client/server combination"):
        client = cx_Oracle.clientversion()[:2]
        if client < minclient:
            self.skipTest(message)
        server = tuple(int(s) for s in self.connection.version.split("."))[:2]
        if server < minserver:
            self.skipTest(message)
        if server > (20, 1) and client < (20, 1):
            self.skipTest(message)
        return self.connection.getSodaDatabase()

    def isOnOracleCloud(self, connection=None):
        if connection is None:
            connection = self.connection
        cursor = connection.cursor()
        cursor.execute("""
                select sys_context('userenv', 'service_name')
                from dual""")
        serviceName, = cursor.fetchone()
        return serviceName.endswith("oraclecloud.com")

    def setUp(self):
        self.connection = GetConnection()
        self.cursor = self.connection.cursor()

    def tearDown(self):
        self.connection.close()
        del self.cursor
        del self.connection




#------------------------------------------------------------------------------
# Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# DropSamples.py
#
# Drops the database objects used for the cx_Oracle samples.
#------------------------------------------------------------------------------

import cx_Oracle
import SampleEnv

def DropSamples(conn):
    print("Dropping sample schemas and edition...")
    SampleEnv.RunSqlScript(conn, "DropSamples",
            main_user = SampleEnv.GetMainUser(),
            edition_user = SampleEnv.GetEditionUser(),
            edition_name = SampleEnv.GetEditionName())

if __name__ == "__main__":
    conn = cx_Oracle.connect(SampleEnv.GetAdminConnectString())
    DropSamples(conn)
    print("Done.")
